﻿----------------------------------------
-- Epic Gem List v 0.2, Copyright 2010 Wolfgang Rufeger, wolfgang@rufeger.de
-- 0.1	Initial version
-- 0.2	Added blue cataclysm gems with test on first load
-- 0.3  Added JC only gems, cataclysm green and WotLK blue gems
-- 0.4	fixed loading of missing gems on first start (delayed GetItemInfo)
-- 0.5	added french translation, thanks to Araldwenn
-- 1.0	most translations from GlobalStrings.lua, quality and level selection, source cleanup, minor fixes
-- 1.1	new design, more detailed selections
-- 1.2	included BC gems
-- 1.3	small bugfixes (2 itemids), added debug mode (/egl debug)
-- 1.4b	added caching DB and options per char in SavedVariables
-- 1.5	show MarketPrice from Auctioneer, if installed
-- 1.6	new custom frame and menu design
----------------------------------------


-- GLOBAL VARIABLES ------------------------------------------------

EGL_VERSION = "ver 1.6 by Ydy @ Shattrath"

EGL_SOCKETSELECT = _G["COLOR"];
EGL_STATSELECT = _G["STATS_LABEL"];
EGL_CLOSE = _G["CLOSE"];
EGL_JC = EGL_GemLocaleText["jc"];

EGL_SCAN_FRAME_TITLE = "Epic Gem List";
EGL_SCAN_FRAME_SUBTITLE = "Retrieving Gem Data from server ...";
EGL_SCAN_FRAME_ABORT = "cancel";
EGL_SCAN_FRAME_INITIALIZING = "Initializing scan ...";
EGL_QUALITY = _G["QUALITY"];
EGL_ITEMLEVEL = _G["LEVEL"];
EGLBUILD = nil;

-- LOCAL VARIABLES ------------------------------------------------
local retries = 0;
local failed = 0;
local waitTable = {};
local waitFrame = nil;
local EGL_Scanned_Gems;

-- INITIALIZE UNSET VARS ------------------------------------------------

if(EGL_Options == nil) then
    EGL_Options = {};
end

if(EGLDB == nil) then
    EGLDB = {};
end

if ( EGL_Options["SelectedStat"]  == nil ) then
    EGL_Options["SelectedStat"] = 0;
end

if ( EGL_Options["SelectedSlot"] == nil ) then
    EGL_Options["SelectedSlot"] = 0;
end

if ( EGL_Options["SelectedQualiOp"] == nil ) then
    EGL_Options["SelectedQualiOp"] = 1;
end

if ( EGL_Options["SelectedQuali"] == nil ) then
    EGL_Options["SelectedQuali"] = 2;
end

if ( EGL_Options["SelectedLevelOp"] == nil ) then
    EGL_Options["SelectedLevelOp"] = 1;
end

if ( EGL_Options["SelectedLevel"] == nil ) then
    EGL_Options["SelectedLevel"] = 2;
end

if (EGL_Options["jc"] == nil ) then
	EGL_Options["jc"] = false;
	EGL_Options["jc_cata"] = false;
end

if (EGL_Options["jc"] and EGL_Options["cataclysm"]) then
	EGL_Options["jc_cata"] = true;
end
if (EGL_Options["jc"] == false) then
	EGL_Options["jc_cata"] = false
end

if ( EGL_Options["buy"]  == nil ) then
    EGL_Options["buy"] = false;
end

if ( EGL_Options["loot"]  == nil ) then
    EGL_Options["loot"] = false;
end

if (EGL_Options["debug"] == nil) then
	EGL_Options["debug"] = false
end

if (EGL_Scanned_Gems == nil) then
	EGL_Scanned_Gems = 0;
end
if (egl_scan_state == nil) then
	egl_scan_state = 0;
end
if (EGL_GemCount == nil) then
	EGL_GemCount = 0;
end
if (egl_shown_gems == nil) then
	egl_shown_gems = 0;
end

-- MISC FUNCTIONS ------------------------------------------------

---- Chat Console Output ----
function out(text)
	DEFAULT_CHAT_FRAME:AddMessage(text)
	--UIErrorsFrame:AddMessage(text, 1.0, 1.0, 0, 1, 10) 
end

---- Delayed script execution ----
function EGL_wait(delay, func, ...)
  if(type(delay)~="number" or type(func)~="function") then
    return false;
  end
  if(waitFrame == nil) then
    waitFrame = CreateFrame("Frame","WaitFrame", UIParent);
    waitFrame:SetScript("onUpdate",function (self,elapse)
      local count = #waitTable;
      local i = 1;
      while(i<=count) do
        local waitRecord = tremove(waitTable,i);
        local d = tremove(waitRecord,1);
        local f = tremove(waitRecord,1);
        local p = tremove(waitRecord,1);
        if(d>elapse) then
          tinsert(waitTable,i,{d-elapse,f,p});
          i = i + 1;
        else
          count = count - 1;
          f(unpack(p));
        end
      end
    end);
  end
  tinsert(waitTable,{delay,func,{...}});
  return true;
end

--- counting table rows with every index format
function egl_tcount(tab)
   local n = #tab
   if (n == 0) then
     for _ in pairs(tab) do
       n = n + 1
     end
   end
   return n
end

--- Market Price from Auctioneer, if installed
function EGL_MarketPrice(itemID)
	if not itemID then
		if EGL_Options["debug"] then
			return "noItemID";
		end;
		return "";
	end
	aucinstalled = false;
	itemName, itemLink, itemQuality, itemLevel, itemTexture = EGL_GemInfo(itemID);
	if AucAdvanced then -- Auctioneer
		sellPrice, numSeen = AucAdvanced.API.GetMarketValue(itemLink);
		aucinstalled = true;
	elseif Atr_GetAuctionBuyout then -- Auctionator
		sellPrice = Atr_GetAuctionBuyout(itemID);
		aucinstalled = true;
	elseif AuctionLite then -- AuctionLite
		local hist = AuctionLite:GetHistoricalPriceById(itemID, 0);
		sellPrice = math.floor(hist.price);
		aucinstalled = true;
	elseif IsAddOnLoaded("AuctionMaster") and (AucMasStatisticVersion and AucMasStatisticVersion.major==1) then -- AuctionMaster
		avgBid, sellPrice, numBid, numBuy = AucMasGetAuctionInfo(itemLink, false);
		aucinstalled = true;	
	end;
	if sellPrice == nil then
		return "";
	else
		sellPrice = tonumber(sellPrice);
	end
	if aucinstalled then
		if (sellPrice ~= nil) and (sellPrice > 0) then
			local g = math.floor(sellPrice/10000);
			local GSC_GOLD = "ff807000"
			local digits = math.floor(math.log10(sellPrice)+1)
			if digits < 9 then
				gsc = string.format("|c%s%5d|r", GSC_GOLD, g)
			else
				gsc = string.format("|c%s%2.1fk|r", GSC_GOLD, g/1000)
			end
			if g == 0 then
				return "";
			end
			return gsc;
		end
	end
	return "";
end

-- LDB BROKER ------------------------------------------------

local ldb = LibStub:GetLibrary("LibDataBroker-1.1",true)
ldb:NewDataObject("Epic Gem List", {
    type = "launcher",
	label = "EpicGemList",
	OnClick = function(_, msg)
        if msg == "LeftButton" then
	        if EGL_Form:IsVisible() then
		        EGL_Form:Hide()
		    else
		        EGL_Form:Show()
	        end
        end
    end,
	icon = "Interface\\AddOns\\EpicGemList\\icon",
	OnTooltipShow = function(tooltip)
		if not tooltip or not tooltip.AddLine then return end
		tooltip:AddLine("Epic Gem List")
	end,
})


-- SLASH HANDLER ------------------------------------------------

function EGL_SlashCommandHandler(msg)
	if (msg == "debug") then
		if EGL_Options["debug"] then
			EGL_Options["debug"] = false;
			out("EGL: Debugging disabled");
		else
			EGL_Options["debug"] = true;
			if (EGL_Form:IsVisible() == false) then
				EGL_Form:Show()
			end
			
			out("EGL: Debugging enabled");
		end
	elseif (msg == "scan") then
		EGL_ForceScan();
		if (EGL_Form:IsVisible() == false) then
			EGL_Form:Show()
		end
	elseif EGL_Form:IsVisible() then
		EGL_Form:Hide()
	else
		EGL_Form:Show()
	end
end

-- ON LOAD ------------------------------------------------

function EGL_OnLoad(this)
	EGL_Form:RegisterEvent("VARIABLES_LOADED");
	eglf = _G["EGL_Form"];
	SLASH_EGL1 = "/epicgemlist";
	SLASH_EGL2 = "/egl";
	SlashCmdList["EGL"] = function(msg)
		EGL_SlashCommandHandler(msg);
	end
	if EGL_Form:IsVisible() then
		EGL_Form:Hide()
	else
		EGL_Form:Show()
	end
	EGL_Options = {};
	EGL_Options["SelectedSlot"] = 0;
	EGL_Options["SelectedStat"] = 0;
	EGL_Options["SelectedQualiOp"] = 1;
	EGL_Options["SelectedQuali"] = 2;
	EGL_Options["SelectedLevelOp"] = 1;
	EGL_Options["SelectedLevel"] = 2;
	EGL_Options["debug"] = false;
	EGL_Scanned_Gems = 0;
	local version, build, builddate, tocversion = GetBuildInfo()
	if ((version == "4.0.1") or (version == "4.0.2")) then
		EGL_Options["cataclysm"] = false;
	else
		EGL_Options["cataclysm"] = true;
	end
	EGL_SlotDropDown_Init();
	EGL_StatDropDown_Init();
end

-- ON SHOW ------------------------------------------------

function EGL_OnShow()
	if EGL_Options["jc"] then
		CheckButtonJC:SetChecked(true);
	else
		CheckButtonJC:SetChecked(false);
	end
end

-- ON EVENT ------------------------------------------------

function EGL_OnEvent(event)
	if (event == "VARIABLES_LOADED") then
		local version, build, builddate, tocversion = GetBuildInfo();
		local mybuild = version..build;
		totalgems = 0;
		egl_bc = egl_tcount(EGL_Data_BC);
		egl_wotlk = egl_tcount(EGL_Data);
		egl_wotlk_jc = egl_tcount(EGL_Data_JC);
		egl_cata = egl_tcount(EGL_Data_Cata);
		egl_cata_jc = egl_tcount(EGL_Data_Cata_JC);
		totalgems = egl_bc + egl_wotlk + egl_wotlk_jc + egl_cata + egl_cata_jc;
		out("EpicGemList loaded ("..totalgems.." gems) ...");
		if (EGLBUILD ~= mybuild) or (EGLBUILD == nil) then
			EGLBUILD = mybuild;
			out("EGL: Scanning gems for new WOW-Client "..version.." (Build "..build.." / "..builddate..")");
			EGL_GemScan();
		end
		if ((version == "4.0.1") or (version == "4.0.2")) then
			EGL_Options["cataclysm"] = false;
		else
			EGL_Options["cataclysm"] = true;
			out("EGL: Cataclysm support enabled");
		end
		if EGL_Options["jc"] then
			CheckButtonJC:SetChecked(true);
		else
			CheckButtonJC:SetChecked(false);
		end
	end
end

-- GET GEM INFO ------------------------------------------------

function EGL_GemInfo(value)
	local iteminfo = {};
	if EGLDB[value] ~= nil then 
		iteminfo = EGLDB[value];
		return iteminfo[1], iteminfo[2], iteminfo[3], iteminfo[4], iteminfo[5];
	else
		EGL_GetGemInfo(value);
		return false;
	end	
end
-- HELP BUTTON MOUSEOVER ------------------------------------------------

function EGL_Help_OnEnter(this)
	local entries = table.getn(EGL_Help);
	EGLTooltip:ClearLines();
	EGLTooltip:SetOwner(this, "ANCHOR_RIGHT", -(this:GetWidth() / 2), 24);
	for i=0,entries do
		EGLTooltip:AddLine(EGL_Help[i]);
	end
    EGLTooltip:Show();
end

function EGL_Help_OnLeave()
  EGLTooltip:Hide();
end

-- ITEM BUTTON FUNCTIONS ------------------------------------------------

function EGLItem_OnEnter(this)
  EGLTooltip:ClearLines();
  local id = this:GetID()+ FauxScrollFrame_GetOffset(EGL_ScrollBar);  
  if id ~= nil then
	local itemName, itemLink, itemQuality, itemLevel, itemTexture = EGL_GemInfo(EGL[id]);
    EGLTooltip:SetOwner(this, "ANCHOR_RIGHT", -(this:GetWidth() / 2), 24);
    EGLTooltip:SetHyperlink("item:"..EGL[id]..":0:0:0");
	EGLTooltip:AddLine("Item Level: "..itemLevel..", ID: "..EGL[id], 0.7,0.7,1);
    EGLTooltip:Show();
  end 
end

function EGLItem_OnLeave()
  SELECTEDITEM = 0;
  EGLTooltip:Hide();
end

function EGLItem_OnClick(this)
	local EGL_Skill_ID;
	local id = this:GetID() + FauxScrollFrame_GetOffset(EGL_ScrollBar);
	local itemName, itemLink, itemQuality, itemLevel, itemTexture = EGL_GemInfo(EGL[id]);

	local editbox = ChatFrameEditBox
	if not editbox then
		editbox = ChatEdit_GetActiveWindow();
    end 
    if AuctionFrame then
        if (AuctionFrame:IsVisible() and CanSendAuctionQuery() and itemName) then
            BrowseName:SetText(itemName);
            AuctionFrameBrowse_Search();
        end
    end
	if (editbox and IsShiftKeyDown()) then
		editbox:Insert(itemLink); 
	end
end


-- RESULT LIST ------------------------------------------------

function EGL_HasNotStat(egltable,value)
	if EGL_Options["SelectedStat"] ~= 0 and 
	 (egltable[value]["stat1"] ~= EGL_Options["SelectedStat"] and egltable[value]["stat2"] ~= EGL_Options["SelectedStat"] and egltable[value]["stat3"] ~= EGL_Options["SelectedStat"]) then
		return true;
	else
		return false;
	end

end

function EGL_IsNotSlot(egltable,value)
	if EGL_Options["SelectedSlot"] ~= 0 and 
	  ((EGL_Options["SelectedSlot"] == 1 and egltable[value]["red"] == false) or
	  (EGL_Options["SelectedSlot"] == 2 and egltable[value]["blue"] == false) or
	  (EGL_Options["SelectedSlot"] == 3 and egltable[value]["yellow"] == false) or
	  (EGL_Options["SelectedSlot"] == 4 and egltable[value]["meta"] == false)) then
		return true;
	else
		return false;
	end
end

function EGL_IsNotQuali(value)
	if EGL_Options["SelectedQuali"] ~= 0 then
		--itemName, itemLink, itemRarity = GetItemInfo(value);
		local itemName, itemLink, itemRarity, itemLevel, itemTexture = EGL_GemInfo(value);
		if itemRarity == nil then
			return false;
		end
		if EGL_Options["SelectedQualiOp"] == 0 then
			if (itemRarity ~= (EGL_Options["SelectedQuali"] + 1)) then
				return true;
			else
				return false;
			end
		elseif EGL_Options["SelectedQualiOp"] == 1 then
			if (itemRarity < (EGL_Options["SelectedQuali"] + 1)) then
				return true;
			else
				return false;
			end
		elseif EGL_Options["SelectedQualiOp"] == 2 then
			if (itemRarity > (EGL_Options["SelectedQuali"] + 1)) then
				return true;
			else
				return false;
			end
		end	
	end
	return false;
end

function EGL_ShowLevel(selected,required)
	if EGL_Options["SelectedLevelOp"] == 0 then
		if selected == required then
			return true;
		else
			return false;
		end
	elseif EGL_Options["SelectedLevelOp"] == 1 then
		if selected <= required then
			return true;
		else
			return false;
		end
	elseif EGL_Options["SelectedLevelOp"] == 2 then
		if selected >= required then
			return true;
		else
			return false;
		end
	end
	return false;
end


function EGL_ScrollBar_Update()
    local line, lineplusoffset, text, extratext;
    local currentgem;    
    EGL = {};    
    currentgem = 1;
	egl_shown_gems = 0;
	
	
	--if ((EGL_Options["SelectedLevel"] == 0) or (EGL_Options["SelectedLevel"] == 3)) then -- ALL, WotLK
	if (EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],2) then -- ALL, WotLK
		for value,_ in pairs(EGL_Data) do    
			additem = true;	  
			if EGL_IsNotQuali(value) then
				additem = false;
			elseif EGL_HasNotStat(EGL_Data,value) then
				additem = false;
			elseif EGL_IsNotSlot(EGL_Data,value) then
				additem = false;
			elseif (EGL_GemInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end
	end
	
	if ((EGL_Options["cataclysm"]) and ((EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],3))) then	-- if 4.0.3 + ALL, Cataclysm
		for value,_ in pairs(EGL_Data_Cata) do    
			additem = true;	    
			if EGL_IsNotQuali(value) then
				additem = false;
			elseif EGL_HasNotStat(EGL_Data_Cata,value) then
				additem = false;
			elseif EGL_IsNotSlot(EGL_Data_Cata,value) then
				additem = false;
			elseif (EGL_GemInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end      
	end
	
	if ((EGL_Options["jc"]) and ((EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],2))) then	-- jc + ALL, WotLK
		for value,_ in pairs(EGL_Data_JC) do    
			additem = true;	    	    
			if EGL_IsNotQuali(value) then
				additem = false;
			elseif EGL_HasNotStat(EGL_Data_JC,value) then
				additem = false;
			elseif EGL_IsNotSlot(EGL_Data_JC,value) then
				additem = false;
			elseif (EGL_GemInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end      
	end

	if ((EGL_Options["jc_cata"]) and ((EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],3))) then	-- if 4.0.3 + ALL, Cata
		for value,_ in pairs(EGL_Data_Cata_JC) do    
			additem = true;	    	    
			if EGL_IsNotQuali(value) then
				additem = false;
			elseif EGL_HasNotStat(EGL_Data_Cata_JC,value) then
				additem = false;
			elseif EGL_IsNotSlot(EGL_Data_Cata_JC,value) then
				additem = false;
			elseif (EGL_GemInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end      
	end	

	if (EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],1) then -- ALL, BC
		for value,_ in pairs(EGL_Data_BC) do    
			additem = true;	  
			if EGL_IsNotQuali(value) then
				additem = false;
			elseif EGL_HasNotStat(EGL_Data_BC,value) then
				additem = false;
			elseif EGL_IsNotSlot(EGL_Data_BC,value) then
				additem = false;
			elseif (EGL_GemInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end
	end
    table.sort(EGL, EGL_SortResults);
    EGL_ShowGemList();    
end

function EGL_SortResults(a, b)
	--local iteminfoA = GetItemInfo(a);
	--local iteminfoB = GetItemInfo(b);
    --local itemNameA, itemLinkA, itemQualityA, itemLevelA = GetItemInfo(a);
    --local itemNameB, itemLinkB, itemQualityB, itemLevelB = GetItemInfo(b);

	local iteminfoA = EGL_GemInfo(a);
	local iteminfoB = EGL_GemInfo(b);
    local itemNameA, itemLinkA, itemQualityA, itemLevelA = EGL_GemInfo(a);
    local itemNameB, itemLinkB, itemQualityB, itemLevelB = EGL_GemInfo(b);
	
	if ((iteminfoA ~= nil) and (iteminfoB ~= nil)) then
		if itemLevelA ~= itemLevelB then	-- Sort by item level first
			return itemLevelA > itemLevelB;
		end
		if itemQualityA ~= itemQualityB then	-- Sort by item quality second
			return itemQualityA > itemQualityB;
		end
		if itemNameA and itemNameB then		-- if same item level, sort by name
			return itemNameA < itemNameB;
		end
	end
	return a < b;						-- sort by id, if no name
end


function EGL_ShowGemList()  
	local i, item, itemm, itemn, itemi;
	local sg = 15;
    MaxItems = table.getn(EGL);
    FauxScrollFrame_Update(EGL_ScrollBar,MaxItems,sg,16);
    for i=1,sg do
		item = _G["EGLItem_"..i];
		itemn = _G["EGLItem_"..i.."_Name"];
		itemi = _G["EGLItem_"..i.."_Icon"];
		itemp = _G["EGLItem_"..i.."_MarketPrice"];
        lineplusoffset = i + FauxScrollFrame_GetOffset(EGL_ScrollBar);
		if lineplusoffset < MaxItems+1 then
			--itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(EGL[lineplusoffset]);
			local itemName, itemLink, itemRarity, itemLevel, itemTexture = EGL_GemInfo(EGL[lineplusoffset]);
			if itemName then
				if itemn ~= nil then
					itemn:SetText(ITEM_QUALITY_COLORS[itemRarity].hex..itemName);
				end
				if itemi ~= nil then
					itemi:SetTexture(itemTexture);
				end
				if itemp ~= nil then
					eglprice = EGL_MarketPrice(EGL[lineplusoffset]);
					if eglprice then
						itemp:SetText(eglprice);
					end
				end
				if item ~= nil then
					item:Show();
				end
			end
		else
			if item ~= nil then
				item:Hide();
			end
		end
    end
	itemm = _G["EGL_GEMCOUNT"];
	if itemm ~= nil then
		itemm:SetText(MaxItems.." gems:");
	end
end

-- INITIAL GEM SCAN ------------------------------------------------

function EGL_GetGemInfoDelayed(value)
	itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(value);
	--iteminfo = GetItemInfo(value);
	if itemName then
		if EGLDB[value] == nil then -- save GemInfo to DB
			EGLDB[value] = { itemName, itemLink, itemRarity, itemLevel, itemTexture };
		end		
		EGL_ScrollBar_Update();
		if EGL_Options["debug"] then
			out("EGL DEBUG: Added delayed Item ID"..value);
		end
	else
		EGL_GetGemInfo(value);
		if EGL_Options["debug"] then
			out("EGL DEBUG: Failed again on Item ID"..value);
		end
	end
end


function EGL_GetGemInfo(value)
	itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(value);
	--iteminfo = GetItemInfo(value);
	if itemName then
		EGL_Scanned_Gems = EGL_Scanned_Gems + 1;
		if EGLDB[value] == nil then -- save GemInfo to DB
			EGLDB[value] = { itemName, itemLink, itemRarity, itemLevel, itemTexture };
		end
		return true;
	else
		EGL_wait(5,EGL_GetGemInfoDelayed,value);
		failed = failed + 1;
		if EGL_Options["debug"] then
			out("EGL DEBUG: Failed to get ItemID "..value);
		end
		return false;
	end
end

function EGL_GemScan()
	EGLDB = {};	
	EGL_Scanned_Gems = 0;
	out("EGL: Scanning gems ...")
	if EGL_Options["cataclysm"] then
		for value,_ in pairs(EGL_Data_Cata) do
			EGL_GetGemInfo(value);
		end
		for value,_ in pairs(EGL_Data_Cata_JC) do
			EGL_GetGemInfo(value);
		end				
	end
	for value,_ in pairs(EGL_Data) do
		EGL_GetGemInfo(value);
	end
	for value,_ in pairs(EGL_Data_JC) do
		EGL_GetGemInfo(value);
	end
	for value,_ in pairs(EGL_Data_BC) do
		EGL_GetGemInfo(value);
	end
	EGL_FinalizeScan();
end

function EGL_AbortScan()
	egl_scan_state=0;
end

function EGL_ForceScan()
	egl_scan_state = 0;
	EGL_GemScan();
end

function EGL_FinalizeScan()
	egl_scan_state=1;
	out("EGL: Scanned "..EGL_Scanned_Gems.." gems.");
	if (failed > 0) then
		out("EGL: Missing "..failed.." gems will be added in a few seconds ...");
		failed = 0;
	end
	EGL_ScrollBar_Update();
end


